// chatdoc.cpp : implementation of the CChatDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "chatter.h"

#include "chatsock.h"
#include "chatdoc.h"
#include "chatvw.h"
#include "sendvw.h"
#include "setupdlg.h"
#include "msg.h"

#ifdef _WIN32
#ifndef _UNICODE
#include <strstrea.h>
#endif
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

BYTE private_key[16]="1234567890123";
BYTE *pSessionKey = private_key;
CNonce Nonce1;
CSendView *pSendView;

extern COnlineUserList* pOnlineUserList;

/////////////////////////////////////////////////////////////////////////////
// CChatDoc

IMPLEMENT_DYNCREATE(CChatDoc, CDocument)

BEGIN_MESSAGE_MAP(CChatDoc, CDocument)
	//{{AFX_MSG_MAP(CChatDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChatDoc construction/destruction

CChatDoc::CChatDoc()
{
	m_bAutoChat = FALSE;
	m_pSocket = NULL;
	m_pFile = NULL;
	m_pArchiveIn = NULL;
	m_pArchiveOut = NULL;
}

CChatDoc::~CChatDoc()
{
}

BOOL CChatDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

#ifdef _WIN32
#ifndef _UNICODE
	if (AfxGetApp()->m_lpCmdLine[0] != '\0')
	{
		TCHAR strHandle[128];
		TCHAR strServer[128];
		int nChannel;

		istrstream(AfxGetApp()->m_lpCmdLine) >> strHandle >> strServer >> nChannel;
		return ConnectSocket(strHandle, strServer, nChannel);
	}
	else
#endif
#endif
	{
		CSetupDlg Dialog;

		Dialog.m_strHandle=m_strHandle;
		Dialog.m_strServer=_T("");
		Dialog.m_nChannel=0;

		while(TRUE)
		{
			if (IDOK != Dialog.DoModal())
				return FALSE;

			if (ConnectSocket(Dialog.m_strHandle, Dialog.m_strServer, Dialog.m_nChannel)){
				CMsg msg;
				m_strHandle=Dialog.m_strHandle;
				m_strServer=Dialog.m_strServer;
				m_strKeyFile=Dialog.m_strKeyFile;
				msg.SetHeader(0,0,m_strServer,m_strHandle);
				Nonce1.Generate();
				msg.m_pNonce=&Nonce1;
				SendMsg(msg);
				return TRUE;
			}

			if (AfxMessageBox(IDS_CHANGEADDRESS,MB_YESNO) == IDNO)
				return FALSE;
		}
	}
}

void CChatDoc::DeleteContents() 
{
	m_bAutoChat = FALSE;

	if ((m_pSocket != NULL) && (m_pFile != NULL) &&	(m_pArchiveOut != NULL))
	{
		CMsg msg;
		CString strTemp;

		if (strTemp.LoadString(IDS_DISCONNECT))
		{
			msg.m_bClose = TRUE;
			msg.SetHeader(10, 3, "end", "end");
			msg.m_strText = m_strHandle + strTemp;
			msg.SecSerialize(*m_pArchiveOut, pSessionKey, 16);
			m_pArchiveOut->Flush();
		}
	}

	delete m_pArchiveOut;
	m_pArchiveOut = NULL;
	delete m_pArchiveIn;
	m_pArchiveIn = NULL;
	delete m_pFile;
	m_pFile = NULL;

	if (m_pSocket != NULL)
	{
		BYTE Buffer[50];
		m_pSocket->ShutDown();

		while(m_pSocket->Receive(Buffer,50) > 0);
	}

	delete m_pSocket;
	m_pSocket = NULL;

	for(POSITION pos=GetFirstViewPosition();pos!=NULL;)
	{
		CView* pView = GetNextView(pos);
		
		if (pView->IsKindOf(RUNTIME_CLASS(CChatView)))
		{
			CChatView* pChatView = (CChatView*)pView;
			pChatView->GetEditCtrl().SetWindowText(_T(""));
		}	
	}
	CDocument::DeleteContents();
}

/////////////////////////////////////////////////////////////////////////////
// CChatDoc Operations

BOOL CChatDoc::ConnectSocket(LPCTSTR lpszHandle, LPCTSTR lpszAddress, UINT nPort)
{
	m_strHandle = lpszHandle;

	m_pSocket = new CChatSocket(this);

	if (!m_pSocket->Create())
	{
		delete m_pSocket;
		m_pSocket = NULL;
		AfxMessageBox(IDS_CREATEFAILED);
		return FALSE;
	}

	while (!m_pSocket->Connect(lpszAddress, nPort + 700))
	{
		if (AfxMessageBox(IDS_RETRYCONNECT,MB_YESNO) == IDNO)
		{
			delete m_pSocket;
			m_pSocket = NULL;
			return FALSE;
		}
	}

	m_pFile = new CSocketFile(m_pSocket);
	CRC6Base *rc6_1, *rc6_2;
	rc6_1 = new CRC6Base;
	rc6_2 = new CRC6Base;
	m_pArchiveIn = new CSecArchive(rc6_1,m_pFile,CArchive::load);
	m_pArchiveOut = new CSecArchive(rc6_2,m_pFile,CArchive::store);

	return TRUE;
}

void CChatDoc::ProcessPendingRead() 
{
	do
	{
		ReceiveMsg();
		if (m_pSocket == NULL)
			return;
	}
	while(!m_pArchiveIn->IsBufferEmpty());
}

void CChatDoc::SendMsg(CString& strText)
{
	if (m_pArchiveOut != NULL)
	{
		CMsg msg;

		msg.SetHeader(10,1, m_strServer, m_strHandle);

		msg.m_strText = m_strHandle + _T(": ") + strText;

		TRY
		{
			msg.SecSerialize(*m_pArchiveOut, pSessionKey, 16);
			m_pArchiveOut->Flush();
		}
		CATCH(CFileException, e)
		{
			m_bAutoChat = FALSE;
			m_pArchiveOut->Abort();
			delete m_pArchiveOut;
			m_pArchiveOut = NULL;
		
			CString strTemp;
			if (strTemp.LoadString(IDS_SERVERRESET))
				DisplayMsg(strTemp);
		}
		END_CATCH
	}
}

void CChatDoc::SendMsg(CMsg& msg)
{
	if (m_pArchiveOut != NULL)
	{
		TRY
		{
			msg.SecSerialize(*m_pArchiveOut, pSessionKey, 16);
			m_pArchiveOut->Flush();
		}
		CATCH(CFileException, e)
		{
			m_bAutoChat = FALSE;
			m_pArchiveOut->Abort();
			delete m_pArchiveOut;
			m_pArchiveOut = NULL;
		
			CString strTemp;
			if (strTemp.LoadString(IDS_SERVERRESET))
				DisplayMsg(strTemp);
		}
		END_CATCH
	}
}

void CChatDoc::ReceiveMsg()
{
	CMsg msg;
	
	TRY
	{
		msg.SecSerialize(*m_pArchiveIn, pSessionKey, 16);
		
		int i;
		int seq=msg.m_header.seq;
		int opt=msg.m_header.opt;
		CString receiver=msg.m_header.receiver;
		CString sender=msg.m_header.sender;

		if(seq==1 && opt==0){

			IUserRSA* pUserRSA;
			HRESULT hr = CoCreateInstance(CLSID_UserRSA,
			NULL, 
			CLSCTX_INPROC_SERVER, 
			IID_IUserRSA, 
			(void **)&pUserRSA);
			if(hr==REGDB_E_CLASSNOTREG){
				AfxMessageBox("REGDB_E_CLASSNOTREG ");
			}
			if(hr==CLASS_E_NOAGGREGATION){
				AfxMessageBox("CLASS_E_NOAGGREGATION ");
			}
			if(FAILED(hr)){
				AfxMessageBox("IUserRSA failed");
				return;
			}
			if(pUserRSA->LoadFile((unsigned char*)(LPCTSTR) m_strKeyFile) != S_OK){
				AfxMessageBox("Private Key File format error!");
				return;
			}
			DWORD key_temp[32];
			DWORD base_temp[32];
			DWORD ans_temp[32];
			int ans=32;
			int base_len, key_len;
			pUserRSA->GetBase(NULL, &base_len);
			pUserRSA->GetBase(base_temp, &base_len);
			pUserRSA->GetPrivateExp(NULL, &key_len);
			pUserRSA->GetPrivateExp(key_temp, &key_len);
			pUserRSA->ExpModPrivate((msg.m_pSessionKey)->m_pdwData, 32, ans_temp, &ans);
			for(i=0; i<32; i++)
				(msg.m_pSessionKey)->m_pdwData[i]=ans_temp[i];
			pUserRSA->Release();

			if( !Nonce1.IsEqual((msg.m_pSessionKey)->m_pdwNonce1) ){
				AfxMessageBox("Private Key Error!");
				return;
			}
			pSessionKey = (msg.m_pSessionKey)->m_pbKey;

			CMsg msg2;
			CNonce nonce2;
			for(i=0; i<8; i++)
				nonce2.m_pdwData[i] = (msg.m_pSessionKey)->m_pdwNonce2[i];
			msg2.m_pNonce = &nonce2;
			msg2.SetHeader(2,0,m_strServer,m_strHandle);
			DisplayMsg(CString("Please wait a moment!"));
			SendMsg(msg2);
			Sleep(1000);
			return;
		}

		if(seq==3 && opt==4){
			CString strTemp;
			pOnlineUserList = msg.m_pUserList;
			strTemp.Format("You have received online user Information!");
			DisplayMsg(strTemp);
			if (strTemp.LoadString(IDS_CONNECT))
				SendMsg(strTemp);
			return;
		}

		if(opt == 2) pOnlineUserList->Add(msg.m_pUser);

		if(opt == 1){

		}

		while(!msg.m_msgList.IsEmpty())
		{
			CString temp = msg.m_msgList.RemoveHead();
			DisplayMsg(temp);
		}

	}
	CATCH(CFileException, e)
	{
		m_bAutoChat = FALSE;
		msg.m_bClose = TRUE;
		m_pArchiveOut->Abort();

		CString strTemp;
		if (strTemp.LoadString(IDS_SERVERRESET))
			DisplayMsg(strTemp);
		if (strTemp.LoadString(IDS_CONNECTIONCLOSED))
			DisplayMsg(strTemp);
	}
	END_CATCH

	if (msg.m_bClose)
	{
		delete m_pArchiveIn;
		m_pArchiveIn = NULL;
		delete m_pArchiveOut;
		m_pArchiveOut = NULL;
		delete m_pFile;
		m_pFile = NULL;
		delete m_pSocket;
		m_pSocket = NULL;
	}
}

void CChatDoc::DisplayMsg(LPCTSTR lpszText)
{

	for(POSITION pos=GetFirstViewPosition();pos!=NULL;)
	{
		CView* pView = GetNextView(pos);
		CChatView* pChatView = DYNAMIC_DOWNCAST(CChatView, pView);
		
		if (pChatView != NULL)
			pChatView->Message(lpszText);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CChatDoc serialization

void CChatDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		for(POSITION pos=GetFirstViewPosition();pos!=NULL;)
		{
			CView* pView = GetNextView(pos);
			CChatView* pChatView = DYNAMIC_DOWNCAST(CChatView, pView);
		
			if (pChatView != NULL)
				pChatView->SerializeRaw(ar);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CChatDoc diagnostics

#ifdef _DEBUG
void CChatDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CChatDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CChatDoc commands

