/*CSecArchive (Secure Archive)

  How-to:

  You may add a member function to your class (serializable class):

	void SecSerialize(CSecArchive &ar, BYTE* key, DWORD keysize){
		ar.Initial(key, keysize);
		Serialize(*(ar.m_parTemp));
		ar.Finish();
 	}

  Then, You can use the Secure Serialization function:

	CCipherBase* pCipher=new CRC5Base();
	CFile* pFile = new CFile(....);
	CSecArchive ar(pCipher, pFile, CArchive::store);

	SecSerialize(ar, key, 16);

	ar.Close();
	pFile->Close();

  Enjoy your Secure Serialization...

  Bee-Chung Chen
*/
#ifndef IncludeThisFileSecArchive
#define IncludeThisFileSecArchive

#include "CipherBase.h"
#include <string.h>

#define DECLARE_SECSERIAL \
		 public:          \
         void SecSerialize(CSecArchive &ar, BYTE* key, DWORD keysize); 


#define IMPLEMENT_SECSERIAL(class_name) \
		void class_name::SecSerialize(CSecArchive &ar, BYTE* key, DWORD keysize){ \
			ar.Initial(key, keysize); \
			Serialize(*(ar.m_parTemp)); \
			ar.Finish(); \
		}

class CSecArchive{

//data members
public:
	CArchive* m_parFile;
	CArchive* m_parTemp;
	CFile* m_pfileTemp;
	DWORD m_dwKey;
	UINT m_nMode;
	int m_nKeySize;    //in byte
	int m_nBlockSize;  //in byte
	int m_nCount;
	int m_nMsgLength; //Length (in byte) of Encrypted Data
	char m_strFileName[20];
	CCipherBase* m_pCipher;

public:
	CSecArchive(CCipherBase* cipher, CFile* pFile, UINT nMode,  
  		        int nBufSize=4096, void* lpBuf=NULL){
		static int ref=0;
		m_parFile=NULL; m_parTemp=NULL; m_pfileTemp=NULL;
		strcpy(m_strFileName,"SecArc00.tmp");
		if(ref >= 10){
			m_strFileName[6] = '0' + (ref%100)/10;
			m_strFileName[7] = '0' + ref % 10;
		}else{
			m_strFileName[7] = '0' + ref;
		}
		m_parFile = new CArchive(pFile,nMode,nBufSize,lpBuf);
		m_nCount=ref;
		m_pCipher=cipher;
		m_nMode=nMode;
		ref++;
	}

	~CSecArchive(void){
		if(m_parFile!=NULL) delete m_parFile;
		if(m_parTemp!=NULL) delete m_parTemp;
		if(m_pfileTemp!=NULL) m_pfileTemp->Close();
		delete m_pfileTemp;
	}

	void Initial(BYTE* pByte, DWORD nKeySize, DWORD dwParamC=0, DWORD dwParamK=0){
		m_pCipher->CipherInitial(dwParamC);
		m_pCipher->KeyInitial(pByte, nKeySize, dwParamK);
		m_nKeySize=nKeySize;
		m_nBlockSize=m_pCipher->GetBlockSize();
		m_pfileTemp = new CFile(m_strFileName, CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

		if(m_nMode == CArchive::load){
			Input();
			m_pfileTemp->Flush();
			m_pfileTemp->SeekToBegin();
		}

		m_parTemp = new CArchive(m_pfileTemp, m_nMode);

	}

	void Finish(void){
		m_parTemp->Close();
		delete m_parTemp;
		m_parTemp=NULL;
		m_pfileTemp->Flush();
		if(m_nMode == CArchive::store){
			Output();
		}
		m_pfileTemp->Close();
		delete m_pfileTemp;
		m_pfileTemp=NULL;
	}

	void Flush(void){
		m_parFile->Flush();
	}

	void Close(void){
		if(m_parFile!=NULL){
			m_parFile->Close();
			delete m_parFile;
			m_parFile = NULL;
		}
		if(m_parTemp!=NULL){
			m_parTemp->Close();
			delete m_parTemp;
			m_parTemp = NULL;
		}
		if(m_pfileTemp!=NULL){
			m_pfileTemp->Close();
			delete m_pfileTemp;
			m_pfileTemp = NULL;
		}
	}

	int IsStoring(void){
		if(m_nMode == CArchive::store) return 1;
		return 0;
	}

	void Abort(void){
		m_parFile->Abort();
	}

	BOOL IsBufferEmpty(void){
		return m_parFile->IsBufferEmpty();
	}

private:
	void Input(void);
	void Output(void);

};

#endif