#ifndef _DATA_H_
#define _DATA_H_

#include "Cdib.h"
#include "UserCrypto_i.c"
#include "UserCrypto.h"
#include <afxtempl.h>
#include "SecArchive.h"

#define KEYDIRECTORY "PUBKEYS"
#define NEWKEYDIR "NEWKEY"
#define USERINFO_FILE "userinfo.bix"
#define LOCAL_KEY "j@e9xOI(&axe.p"

CString ToString(DWORD *pData, int len);
void Randomize(DWORD *pData, int len);


class CUserInfo : public CObject{
	
	DECLARE_SERIAL(CUserInfo)
	DECLARE_SECSERIAL

public:
//data member
	CString m_strID;
	CString m_strName;
	int m_nMale;
	CString m_strOthers;
	CDib m_dibPic;

// member functions
	virtual void Serialize(CArchive& ar);

	int SetInfo(CString ID, CString Name, int Male, CString Others, CString bmp);

};



class CUserList : public CObject{

	DECLARE_SERIAL(CUserList)
    DECLARE_SECSERIAL

public:
//data member
	CMapStringToPtr m_mapData;

private:
	POSITION ptr;

public:

	CUserList(void){
		ptr=NULL;
	}

	~CUserList(void);

	void Add(CUserInfo* ui){
		if(Find(ui->m_strID)!=NULL){
			AfxMessageBox("The Same ID Found!! New ID Please!!");
			return;
		}
		m_mapData.SetAt(ui->m_strID, ui);
	}

	CUserInfo* Find(CString id){
		void* result;
		int tmp=m_mapData.Lookup(id, result);
		if(tmp == 0) return NULL;
		return (CUserInfo*) result;
	}

	CUserInfo* First(void){
		ptr=m_mapData.GetStartPosition();
		if( ptr==NULL) return NULL;
		void* ui;
		CString str;
		m_mapData.GetNextAssoc(ptr, str, ui);
		return (CUserInfo*) ui;
	}

	CUserInfo* Next(void){
		if(ptr==NULL) return NULL;
		void* ui;
		CString str;
		m_mapData.GetNextAssoc(ptr,str,ui);
		return (CUserInfo*) ui;
	}

	int GetCount(void){
		return m_mapData.GetCount();
	}

	// return 0 for error
	int Remove(CString id){
		CUserInfo *ui;
		ui = Find(id);
		if(ui==NULL) return 0;
		delete ui;
		return m_mapData.RemoveKey(id);
	}

	int IsEmpty(void){
		return m_mapData.IsEmpty();
	}

//member functions
	virtual void Serialize(CArchive& ar);
};


class COnlineUser : public CObject{
	
	DECLARE_SERIAL(COnlineUser)
	DECLARE_SECSERIAL

public:
	int m_nIsNew;
	CUserInfo* m_pUserInfo;
	DWORD m_pdwBase[32];
	DWORD m_pdwPublicKey[32];
	int m_nBaseSize;
	int m_nKeySize;

public:

	COnlineUser(void){
		m_pUserInfo=NULL;
		m_nIsNew=0;
	}

	~COnlineUser(void){
		if(m_nIsNew==1){
			delete m_pUserInfo;
		}
	}

	void SetInfo(CUserInfo* ui, DWORD key[32], int key_size, DWORD base[32], int base_size);
	virtual void Serialize(CArchive& ar);
};


class COnlineUserList : public CObject{

	DECLARE_SERIAL(COnlineUserList)
    DECLARE_SECSERIAL

public:
//data member
	CMapStringToPtr m_mapData;

private:
	POSITION ptr;

public:

	COnlineUserList(void){
		ptr=NULL;
	}

	~COnlineUserList(void);

	void Add(COnlineUser* ou){
		/*if(Find((ou->m_pUserInfo)->m_strID)!=NULL){
			AfxMessageBox("The Same ID Found!! New ID Please!!");
			return;
		}*/
		m_mapData.SetAt((ou->m_pUserInfo)->m_strID, ou);
	}

	COnlineUser* Find(CString id){
		void* result;
		int tmp=m_mapData.Lookup(id, result);
		if(tmp == 0) return NULL;
		return (COnlineUser*) result;
	}

	COnlineUser* First(void){
		ptr=m_mapData.GetStartPosition();
		if( ptr==NULL) return NULL;
		void* ou;
		CString str;
		m_mapData.GetNextAssoc(ptr, str, ou);
		return (COnlineUser*) ou;
	}

	COnlineUser* Next(void){
		if(ptr==NULL) return NULL;
		void* ou;
		CString str;
		m_mapData.GetNextAssoc(ptr,str,ou);
		return (COnlineUser*) ou;
	}

	int GetCount(void){
		return m_mapData.GetCount();
	}

	// return 0 for error
	int Remove(CString id){
		COnlineUser *ou;
		ou = Find(id);
		if(ou==NULL) return 0;
		delete ou;
		return m_mapData.RemoveKey(id);
	}

	int IsEmpty(void){
		return m_mapData.IsEmpty();
	}

//member functions
	virtual void Serialize(CArchive& ar);
};


class CNonce : public CObject{

	DECLARE_SERIAL(CNonce)

public:
	
	DWORD m_pdwData[8];

	void Generate(void){
		Randomize(m_pdwData, 8);
	}

	int IsEqual(DWORD data[8]);

	void CopyTo(DWORD data[8]);

	virtual void Serialize(CArchive& ar);
};

class CSessionKey : public CObject{

	DECLARE_SERIAL(CSessionKey)

public:

	BYTE* m_pbKey;
	DWORD* m_pdwNonce1;
	DWORD* m_pdwNonce2;

	DWORD m_pdwData[32];

	CSessionKey(void){
		m_pdwData[0]=0;
		m_pdwData[1]=0;
		m_pbKey = (BYTE*) (&(m_pdwData[2]));
		m_pdwNonce1 = &(m_pdwData[8]);
		m_pdwNonce2 = &(m_pdwData[16]);
		Randomize(&(m_pdwData[24]), 7);
		m_pdwData[31]=0;
	}

	void SetKey(BYTE key[16]);
	void SetNonce1(DWORD nonce1[8]);
	void GenNonce2(void){
		Randomize(m_pdwNonce2, 8);
	}
	//1 : equal , 0 : otherwise
	int CompareNonce1(DWORD nonce[8]);

	void CopyKeyTo(BYTE key[16]);
	void CopyNonce2To(DWORD nonce[8]);

	void Encrypt(DWORD key[32], int key_len, DWORD base[32], int base_len);
	void Decrypt(DWORD key[32], int key_len, DWORD base[32], int base_len);

	virtual void Serialize(CArchive& ar);
};

class CSentance : public CObject{

	DECLARE_SERIAL(CSentance)
	DECLARE_SECSERIAL

//data member
public:

	CString m_strContent;
	DWORD m_pdwSignature[32];

//member function
public:

	void SetContent(CString content){
		m_strContent = content;
	}

	void Sign(DWORD key[32], int key_len, DWORD base[32], int base_len);

	int Verify(DWORD key[32],int key_len, DWORD base[32], int base_len);

	CString GetSignature(void){
		return ToString(m_pdwSignature, 32);
	}

	virtual void Serialize(CArchive& ar);
};


/*
class CKeyList : CObject{

//data members
	CMapStringToPtr m_mapKeys;
	POSITION ptr;

public:

	CKeyList(void){
		ptr=NULL;
	}

	~CKeyList(void);

	void Add(CString ID, IUserRSA* rsa){
		if(Find(ID)!=NULL){
			AfxMessageBox("The Same ID Found!!");
			return;
		}
		m_mapKeys.SetAt(ID, rsa);
	}

	IUserRSA* Find(CString id){
		void* result;
		int tmp=m_mapKeys.Lookup(id, result);
		if(tmp == 0) return NULL;
		return (IUserRSA*) result;
	}

	IUserRSA* First(void){
		ptr=m_mapKeys.GetStartPosition();
		if( ptr==NULL) return NULL;
		void* key;
		CString str;
		m_mapKeys.GetNextAssoc(ptr, str, key);
		return (IUserRSA*) key;
	}

	IUserRSA* Next(void){
		if(ptr==NULL) return NULL;
		void* key;
		CString str;
		m_mapKeys.GetNextAssoc(ptr,str,key);
		return (IUserRSA*) key;
	}

	int GetCount(void){
		return m_mapKeys.GetCount();
	}

	// return 0 for error
	int Remove(CString id){
		IUserRSA* rsa;
		rsa = Find(id);
		if(rsa==NULL) return 0;
		rsa->Release();
		return m_mapKeys.RemoveKey(id);
	}

};
*/
#endif